@@ -1,6 +1,8 @@
 /*
- * Copyright (C) 2003-2025 Sébastien Helleu <flashcode@flashtux.org>
- * Copyright (C) 2005-2006 Emmanuel Bouthenot <kolter@openics.org>
+ * SPDX-FileCopyrightText: 2003-2025 Sébastien Helleu <flashcode@flashtux.org>
+ * SPDX-FileCopyrightText: 2005-2006 Emmanuel Bouthenot <kolter@openics.org>
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -352,6 +354,7 @@ extern struct t_config_option *config_color_status_time;
 extern struct t_config_option *config_completion_base_word_until_cursor;
 extern struct t_config_option *config_completion_case_sensitive;
 extern struct t_config_option *config_completion_command_inline;
+extern struct t_config_option *config_completion_cycle;
 extern struct t_config_option *config_completion_default_template;
 extern struct t_config_option *config_completion_nick_add_space;
 extern struct t_config_option *config_completion_nick_case_sensitive;
@@ -360,6 +363,7 @@ extern struct t_config_option *config_completion_nick_first_only;
 extern struct t_config_option *config_completion_nick_ignore_chars;
 extern struct t_config_option *config_completion_nick_ignore_words;
 extern struct t_config_option *config_completion_partial_completion_alert;
+extern struct t_config_option *config_completion_partial_completion_auto_expand;
 extern struct t_config_option *config_completion_partial_completion_command;
 extern struct t_config_option *config_completion_partial_completion_command_arg;
 extern struct t_config_option *config_completion_partial_completion_count;
