@@ -1,7 +1,7 @@
 /*
- * core-util.c - some useful functions
+ * SPDX-FileCopyrightText: 2003-2025 Sébastien Helleu <flashcode@flashtux.org>
  *
- * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -19,6 +19,8 @@
  * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+/* Some useful functions */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -27,6 +29,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
+#include <limits.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #ifdef HAVE_SYS_RESOURCE_H
@@ -46,6 +49,118 @@
 #include "../plugins/plugin.h"
 
 
+/*
+ * Parses an integer.
+ *
+ * If result is not NULL, *result is set with the parsed integer in case
+ * there is no error.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+util_parse_int (const char *string, int base, int *result)
+{
+    long number;
+    char *error;
+
+    if (!string || !string[0])
+        return 0;
+
+    /* base must be 0 or between 2-36 (inclusive) */
+    if ((base < 0) || (base == 1) || (base > 36))
+        return 0;
+
+    errno = 0;
+    error = NULL;
+    number = strtol (string, &error, base);
+
+    if ((errno == ERANGE) || !error || error[0] || (error == string)
+        || (number < INT_MIN) || (number > INT_MAX))
+        return 0;
+
+    if (result)
+        *result = (int)number;
+
+    return 1;
+}
+
+/*
+ * Parses a long integer.
+ *
+ * If result is not NULL, *result is set with the parsed long integer in case
+ * there is no error.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+util_parse_long (const char *string, int base, long *result)
+{
+    long number;
+    char *error;
+
+    if (!string || !string[0])
+        return 0;
+
+    /* base must be 0 or between 2-36 (inclusive) */
+    if ((base < 0) || (base == 1) || (base > 36))
+        return 0;
+
+    errno = 0;
+    error = NULL;
+    number = strtol (string, &error, base);
+
+    if ((errno == ERANGE) || !error || error[0] || (error == string))
+        return 0;
+
+    if (result)
+        *result = number;
+
+    return 1;
+}
+
+/*
+ * Parses a long long integer.
+ *
+ * If result is not NULL, *result is set with the parsed long integer in case
+ * there is no error.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+util_parse_longlong (const char *string, int base, long long *result)
+{
+    long long number;
+    char *error;
+
+    if (!string || !string[0])
+        return 0;
+
+    /* base must be 0 or between 2-36 (inclusive) */
+    if ((base < 0) || (base == 1) || (base > 36))
+        return 0;
+
+    errno = 0;
+    error = NULL;
+    number = strtoll (string, &error, base);
+
+    if ((errno == ERANGE) || !error || error[0] || (error == string))
+        return 0;
+
+    if (result)
+        *result = number;
+
+    return 1;
+}
+
 /*
  * Compares two timeval structures.
  *
@@ -123,9 +238,9 @@ util_timeval_add (struct timeval *tv, long long interval)
  */
 
 char *
-util_get_microseconds_string (long long microseconds)
+util_get_microseconds_string (unsigned long long microseconds)
 {
-    long long hour, min, sec, usec;
+    unsigned long long hour, min, sec, usec;
     char result[128];
 
     usec = microseconds % 1000000;
@@ -134,7 +249,7 @@ util_get_microseconds_string (long long microseconds)
     hour = (microseconds / 1000000) / 3600;
 
     snprintf (result, sizeof (result),
-              "%lld:%02lld:%02lld.%06lld",
+              "%llu:%02llu:%02llu.%06llu",
               hour, min, sec, usec);
 
     return strdup (result);
@@ -166,6 +281,8 @@ util_get_time_string (const time_t *date)
 /*
  * Formats date and time like strftime (but with timeval structure as input)
  * and adds extra specifiers:
+ *   - "%@": return the date expressed in Coordinated Universal Time (UTC)
+ *           instead of date relative to the user's specified timezone
  *   - "%.1" to "%.6": first N digits of microseconds, zero-padded
  *   - "%f": alias of "%.6" (microseconds, zero-padded to 6 digits)
  *   - "%!": timestamp as integer, in seconds (value of tv->tv_sec)
@@ -176,13 +293,15 @@ util_strftimeval (char *string, int max, const char *format, struct timeval *tv)
 {
     char **format2, str_temp[32];
     const char *ptr_format;
-    struct tm *local_time;
-    int length, bytes;
+    struct tm *date_time;
+    int length, bytes, local_time;
+    long usec;
 
     if (!string || (max <= 0) || !format || !tv)
         return 0;
 
     string[0] = '\0';
+    local_time = 1;
 
     if (!format[0])
         return 0;
@@ -191,6 +310,12 @@ util_strftimeval (char *string, int max, const char *format, struct timeval *tv)
     if (!format2)
         return 0;
 
+    usec = (long)(tv->tv_usec);
+    if (usec < 0)
+        usec = 0;
+    else if (usec > 999999)
+        usec = 999999;
+
     ptr_format = format;
     while (ptr_format && ptr_format[0])
     {
@@ -199,12 +324,16 @@ util_strftimeval (char *string, int max, const char *format, struct timeval *tv)
             string_dyn_concat (format2, "%%", -1);
             ptr_format += 2;
         }
+        else if ((ptr_format[0] == '%') && (ptr_format[1] == '@'))
+        {
+            local_time = 0;
+            ptr_format += 2;
+        }
         else if ((ptr_format[0] == '%') && (ptr_format[1] == '.'))
         {
             if ((ptr_format[2] >= '1') && (ptr_format[2] <= '6'))
             {
-                snprintf (str_temp, sizeof (str_temp),
-                          "%06ld", (long)(tv->tv_usec));
+                snprintf (str_temp, sizeof (str_temp), "%06ld", usec);
                 length = ptr_format[2] - '1' + 1;
                 str_temp[length] = '\0';
                 string_dyn_concat (format2, str_temp, -1);
@@ -219,13 +348,13 @@ util_strftimeval (char *string, int max, const char *format, struct timeval *tv)
         }
         else if ((ptr_format[0] == '%') && (ptr_format[1] == 'f'))
         {
-            snprintf (str_temp, sizeof (str_temp), "%06ld", (long)(tv->tv_usec));
+            snprintf (str_temp, sizeof (str_temp), "%06ld", usec);
             string_dyn_concat (format2, str_temp, -1);
             ptr_format += 2;
         }
         else if ((ptr_format[0] == '%') && (ptr_format[1] == '!'))
         {
-            snprintf (str_temp, sizeof (str_temp), "%ld", (long)(tv->tv_sec));
+            snprintf (str_temp, sizeof (str_temp), "%lld", (long long)(tv->tv_sec));
             string_dyn_concat (format2, str_temp, -1);
             ptr_format += 2;
         }
@@ -236,14 +365,17 @@ util_strftimeval (char *string, int max, const char *format, struct timeval *tv)
         }
     }
 
-    local_time = localtime (&(tv->tv_sec));
-    if (!local_time)
+    if (local_time)
+        date_time = localtime (&(tv->tv_sec));
+    else
+        date_time = gmtime (&(tv->tv_sec));
+    if (!date_time)
     {
         string_dyn_free (format2, 1);
         return 0;
     }
 
-    bytes = strftime (string, max, *format2, local_time);
+    bytes = strftime (string, max, *format2, date_time);
 
     string_dyn_free (format2, 1);
 
@@ -252,24 +384,29 @@ util_strftimeval (char *string, int max, const char *format, struct timeval *tv)
 
 /*
  * Parses a date/time string, which can be one of these formats:
- *   "2024-01-04"                  -> date at midnight
- *   "2024-01-04T22:01:02"         -> ISO 8601, local time
- *   "2024-01-04T22:01:02.123"     -> ISO 8601, local time, with milliseconds
- *   "2024-01-04T22:01:02.123456"  -> ISO 8601, local time, with microseconds
- *   "2024-01-04T21:01:02Z"        -> ISO 8601, UTC
- *   "2024-01-04T21:01:02.123Z"    -> ISO 8601, UTC, with milliseconds
- *   "2024-01-04T21:01:02.123456Z" -> ISO 8601, UTC, with microseconds
- *   "22:01:02"                    -> current date, local time
- *   "22:01:02.123"                -> current date, local time with milliseconds
- *   "22.01:02.123456"             -> current date, local time with microseconds
- *   "21:01:02Z"                   -> current date, UTC
- *   "21:01:02.123Z"               -> current date, UTC, with milliseconds
- *   "21.01:02.123456Z"            -> current date, UTC, with microseconds
- *   "1704402062"                  -> timestamp date
- *   "1704402062.123"              -> timestamp date, with milliseconds
- *   "1704402062,123"              -> timestamp date, with milliseconds
- *   "1704402062.123456"           -> timestamp date, with microseconds
- *   "1704402062,123456"           -> timestamp date, with microseconds
+ *
+ *   Format                               | Example
+ *   -------------------------------------+----------------------------------
+ *   Date (midnight)                      | 2025-08-30
+ *   ISO 8601, date + local time          | 2025-08-30T21:04:55.123456
+ *   ISO 8601, date + local time + offset | 2025-08-30T21:04:55.123456+0200
+ *   ISO 8601, date + UTC time            | 2025-08-30T19:04:55.123456Z
+ *   RFC 3339, date + local time          | 2025-08-30 21:04:55.123456
+ *   RFC 3339, date + local time + offset | 2025-08-30 21:04:55.123456 +02:00
+ *   RFC 3339, date + UTC time            | 2025-08-30 19:04:55.123456Z
+ *   Local time                           | 21:04:55.123456
+ *   Local time + offset                  | 21:04:55.123456+0200
+ *   UTC time                             | 19:04:55.123456Z
+ *   Timestamp date                       | 1756580695.123456
+ *
+ * Notes :
+ *
+ * - For ISO 8601, characters `T' and `Z` can be used in lower case: `t` and `z`.
+ * - The timezone offset format is `[+/-]hh:mm`, `[+/-]hhmm` or `[+/-]hh`.
+ * - Precision after seconds can be from one-tenth of second (1 digit, like
+ *   `21:04:55.1`) to one microsecond (6 digits, like `21:04:55.123456`).
+ *   A dot (`.`) or comma (`,`) can be used to separate seconds from the other
+ *   digits.
  *
  * Returns:
  *   1: OK
@@ -279,10 +416,11 @@ util_strftimeval (char *string, int max, const char *format, struct timeval *tv)
 int
 util_parse_time (const char *datetime, struct timeval *tv)
 {
-    char *string, *pos, *pos2, str_usec[16], *error, str_date[128];
+    char *string, *pos, *pos2, *pos_colon, *pos_hyphen, *pos_dot;
+    char str_usec[16], *error, str_date[128], str_date2[256];
     struct tm tm_date, tm_date_gm, tm_date_local, *local_time;
     time_t time_now, time_gm, time_local;
-    long value;
+    long long value;
     int rc, length, use_local_time, timezone_offset, offset_factor, hour, min;
 
     if (!datetime || !datetime[0] || !tv)
@@ -297,11 +435,36 @@ util_parse_time (const char *datetime, struct timeval *tv)
     timezone_offset = 0;
     offset_factor = 1;
 
-    string = strdup (datetime);
-    if (!string)
-        return 0;
+    pos_colon = strchr (datetime, ':');
+    pos_hyphen = strchr (datetime, '-');
+    pos_dot = strchr (datetime, '.');
+    if (pos_colon && !pos_hyphen)
+    {
+        /* add current date: "19:04:55" -> "2025-08-30T19:04:55" */
+        string = malloc (strlen (datetime) + 16 + 1);
+        if (!string)
+            return 0;
+        time_now = time (NULL);
+        local_time = localtime (&time_now);
+        strftime (str_date, sizeof (str_date), "%Y-%m-%dT", local_time);
+        snprintf (string, sizeof (str_date2), "%s%s", str_date, datetime);
+    }
+    else if (!pos_colon && pos_hyphen && (!pos_dot || (pos_hyphen < pos_dot)))
+    {
+        /* add time (midnight): "2025-08-30" -> "2025-08-30T00:00:00" */
+        string = malloc (strlen (datetime) + 16 + 1);
+        if (!string)
+            return 0;
+        snprintf (string, sizeof (str_date2), "%sT00:00:00", datetime);
+    }
+    else
+    {
+        string = strdup (datetime);
+        if (!string)
+            return 0;
+    }
 
-    /* extract microseconds and remove them from string2 */
+    /* extract microseconds and remove them from string */
     pos = strchr (string, '.');
     if (!pos)
         pos = strchr (string, ',');
@@ -324,21 +487,27 @@ util_parse_time (const char *datetime, struct timeval *tv)
                 strcat (str_usec, "0");
             }
             error = NULL;
-            value = strtol (str_usec, &error, 10);
+            value = strtoll (str_usec, &error, 10);
             if (error && !error[0])
             {
+                /*
+                 * just in case: this should not happen as minus is not
+                 * supported and we truncate at 6 digits
+                 */
                 if (value < 0)
                     value = 0;
                 else if (value > 999999)
                     value = 999999;
-                tv->tv_usec = (int)value;
+                tv->tv_usec = (long)value;
             }
         }
         memmove (pos, pos2, strlen (pos2) + 1);
     }
 
-    /* extract timezone and remove it from string2 */
-    pos = strchr (string, 'Z');
+    /* extract timezone and remove it from string */
+    pos = strrchr (string, 'Z');
+    if (!pos)
+        pos = strrchr (string, 'z');
     if (pos)
     {
         pos[0] = '\0';
@@ -348,9 +517,13 @@ util_parse_time (const char *datetime, struct timeval *tv)
     else
     {
         pos = strchr (string, 'T');
+        if (!pos)
+            pos = strchr (string, 't');
+        if (!pos)
+            pos = strchr (string, ' ');
         if (pos)
         {
-            pos2 = strchr (pos, '+');
+            pos2 = strrchr (pos, '+');
             if (pos2)
             {
                 pos2[0] = '\0';
@@ -359,7 +532,7 @@ util_parse_time (const char *datetime, struct timeval *tv)
             }
             else
             {
-                pos2 = strchr (pos, '-');
+                pos2 = strrchr (pos, '-');
                 if (pos2)
                 {
                     pos2[0] = '\0';
@@ -390,11 +563,11 @@ util_parse_time (const char *datetime, struct timeval *tv)
         }
     }
 
-    if (strchr (string, '-'))
+    if (strchr (string, '-') && strchr (string, ':'))
     {
-        if (strchr (string, ':'))
+        if (strchr (string, 'T'))
         {
-            /* ISO 8601 format like: "2024-01-04T21:01:02.123Z" */
+            /* ISO 8601 format like: "2025-08-30T19:04:55" */
             /* initialize structure, because strptime does not do it */
             memset (&tm_date, 0, sizeof (struct tm));
             pos = strptime (string, "%Y-%m-%dT%H:%M:%S", &tm_date);
@@ -402,6 +575,7 @@ util_parse_time (const char *datetime, struct timeval *tv)
             {
                 if (use_local_time)
                 {
+                    tm_date.tm_isdst = -1;
                     tv->tv_sec = mktime (&tm_date);
                 }
                 else
@@ -414,61 +588,73 @@ util_parse_time (const char *datetime, struct timeval *tv)
                     time_local = mktime (&tm_date_local);
                     tv->tv_sec = mktime (&tm_date_local)
                         + (time_local - time_gm)
-                        + timezone_offset;
+                        - timezone_offset;
+                }
+                rc = 1;
+            }
+        }
+        else if (strchr (string, 't'))
+        {
+            /* ISO 8601 format like: "2025-08-30t19:04:55" */
+            /* initialize structure, because strptime does not do it */
+            memset (&tm_date, 0, sizeof (struct tm));
+            pos = strptime (string, "%Y-%m-%dt%H:%M:%S", &tm_date);
+            if (pos && (tm_date.tm_year > 0))
+            {
+                if (use_local_time)
+                {
+                    tm_date.tm_isdst = -1;
+                    tv->tv_sec = mktime (&tm_date);
+                }
+                else
+                {
+                    /* convert to UTC and add timezone_offset */
+                    time_now = mktime (&tm_date);
+                    gmtime_r (&time_now, &tm_date_gm);
+                    localtime_r (&time_now, &tm_date_local);
+                    time_gm = mktime (&tm_date_gm);
+                    time_local = mktime (&tm_date_local);
+                    tv->tv_sec = mktime (&tm_date_local)
+                        + (time_local - time_gm)
+                        - timezone_offset;
                 }
                 rc = 1;
             }
         }
         else
         {
-            /* ISO 8601 format like: "2024-01-04" */
+            /* like ISO 8601 but with space like: "2025-08-30 19:04:55" */
             /* initialize structure, because strptime does not do it */
             memset (&tm_date, 0, sizeof (struct tm));
-            pos = strptime (string, "%Y-%m-%d", &tm_date);
+            pos = strptime (string, "%Y-%m-%d %H:%M:%S", &tm_date);
             if (pos && (tm_date.tm_year > 0))
             {
-                tv->tv_sec = mktime (&tm_date);
+                if (use_local_time)
+                {
+                    tm_date.tm_isdst = -1;
+                    tv->tv_sec = mktime (&tm_date);
+                }
+                else
+                {
+                    /* convert to UTC and add timezone_offset */
+                    time_now = mktime (&tm_date);
+                    gmtime_r (&time_now, &tm_date_gm);
+                    localtime_r (&time_now, &tm_date_local);
+                    time_gm = mktime (&tm_date_gm);
+                    time_local = mktime (&tm_date_local);
+                    tv->tv_sec = mktime (&tm_date_local)
+                        + (time_local - time_gm)
+                        - timezone_offset;
+                }
                 rc = 1;
             }
         }
     }
-    else if (strchr (string, ':'))
-    {
-        /* hour format like: "21:01:02" */
-        time_now = time (NULL);
-        local_time = localtime (&time_now);
-        strftime (str_date, sizeof (str_date),
-                  "%Y-%m-%dT", local_time);
-        strcat (str_date, string);
-        /* initialize structure, because strptime does not do it */
-        memset (&tm_date, 0, sizeof (struct tm));
-        pos = strptime (str_date, "%Y-%m-%dT%H:%M:%S", &tm_date);
-        if (pos)
-        {
-            if (use_local_time)
-            {
-                tv->tv_sec = mktime (&tm_date);
-            }
-            else
-            {
-                /* convert to UTC and add timezone_offset */
-                time_now = mktime (&tm_date);
-                gmtime_r (&time_now, &tm_date_gm);
-                localtime_r (&time_now, &tm_date_local);
-                time_gm = mktime (&tm_date_gm);
-                time_local = mktime (&tm_date_local);
-                tv->tv_sec = mktime (&tm_date_local)
-                    + (time_local - time_gm)
-                    + timezone_offset;
-            }
-            rc = 1;
-        }
-    }
     else
     {
         /* timestamp format: "1704402062" */
         error = NULL;
-        value = strtol (string, &error, 10);
+        value = strtoll (string, &error, 10);
         if (error && !error[0] && (value >= 0))
         {
             tv->tv_sec = (time_t)value;
@@ -534,18 +720,26 @@ util_get_time_diff (time_t time1, time_t time2,
  *   - 60000000: minutes
  *   - 3600000000: hours
  *
- * Returns the delay in microseconds, -1 if error.
+ * Returns:
+ *   1: OK
+ *   0: error
  */
 
-long long
-util_parse_delay (const char *string_delay, long long default_factor)
+int
+util_parse_delay (const char *string_delay, unsigned long long default_factor,
+                  unsigned long long *delay)
 {
     const char *pos;
     char *str_number, *error;
-    long long factor, delay;
+    unsigned long long factor;
+
+    if (!delay)
+        return 0;
+
+    *delay = 0;
 
     if (!string_delay || !string_delay[0] || (default_factor < 1))
-        return -1LL;
+        return 0;
 
     factor = default_factor;
 
@@ -557,39 +751,44 @@ util_parse_delay (const char *string_delay, long long default_factor)
 
     if ((pos > string_delay) && pos[0])
     {
-        str_number = string_strndup (string_delay, pos - string_delay);
         if (strcmp (pos, "us") == 0)
-            factor = 1LL;
+            factor = 1ULL;
         else if (strcmp (pos, "ms") == 0)
-            factor = 1000LL;
+            factor = 1000ULL;
         else if (strcmp (pos, "s") == 0)
-            factor = 1000LL * 1000LL;
+            factor = 1000ULL * 1000ULL;
         else if (strcmp (pos, "m") == 0)
-            factor = 1000LL * 1000LL * 60LL;
+            factor = 1000ULL * 1000ULL * 60ULL;
         else if (strcmp (pos, "h") == 0)
-            factor = 1000LL * 1000LL * 60LL * 60LL;
+            factor = 1000ULL * 1000ULL * 60ULL * 60ULL;
         else
-            return -1LL;
+            return 0;
+        str_number = string_strndup (string_delay, pos - string_delay);
     }
     else
     {
+        if (string_delay[0] == '-')
+            return 0;
         str_number = strdup (string_delay);
     }
 
     if (!str_number)
-        return -1LL;
+        return 0;
 
     error = NULL;
-    delay = strtoll (str_number, &error, 10);
-    if (!error || error[0] || (delay < 0))
+    *delay = strtoull (str_number, &error, 10);
+    if (!error || error[0])
     {
         free (str_number);
-        return -1LL;
+        *delay = 0;
+        return 0;
     }
 
+    *delay *= factor;
+
     free (str_number);
 
-    return delay * factor;
+    return 1;
 }
 
 /*
@@ -608,13 +807,16 @@ util_parse_delay (const char *string_delay, long long default_factor)
  *   "0.3.0"     ==> 196608   (== 0x00030000)
  */
 
-int
+unsigned long
 util_version_number (const char *version)
 {
     char **items, buf[64], *error;
     const char *ptr_item;
-    int num_items, i, version_int[4], index_buf;
-    long number;
+    int num_items, i, index_buf;
+    unsigned long number, values[4];
+
+    if (!version || !version[0])
+        return 0;
 
     items = string_split (version, ".", NULL,
                           WEECHAT_STRING_SPLIT_STRIP_LEFT
@@ -623,7 +825,7 @@ util_version_number (const char *version)
                           4, &num_items);
     for (i = 0; i < 4; i++)
     {
-        version_int[i] = 0;
+        values[i] = 0;
         if (items && (i < num_items))
         {
             ptr_item = items[i];
@@ -643,20 +845,18 @@ util_version_number (const char *version)
             if (buf[0])
             {
                 error = NULL;
-                number = strtol (buf, &error, 10);
+                number = strtoul (buf, &error, 10);
                 if (error && !error[0])
                 {
-                    if (number < 0)
-                        number = 0;
-                    else if (number > 0xFF)
+                    if (number > 0xFF)
                         number = 0xFF;
-                    version_int[i] = number;
+                    values[i] = number;
                 }
             }
         }
     }
     string_free_split (items);
 
-    return (version_int[0] << 24) | (version_int[1] << 16)
-        | (version_int[2] << 8) | version_int[3];
+    return (values[0] << 24) | (values[1] << 16)
+        | (values[2] << 8) | values[3];
 }
