@@ -1,5 +1,7 @@
 /*
- * Copyright (C) 2003-2022 Sébastien Helleu <flashcode@flashtux.org>
+ * SPDX-FileCopyrightText: 2003-2025 Sébastien Helleu <flashcode@flashtux.org>
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -20,6 +22,6 @@
 #ifndef WEECHAT_PLUGIN_LOGGER_INFO_H
 #define WEECHAT_PLUGIN_LOGGER_INFO_H
 
-extern void logger_info_init ();
+extern void logger_info_init (void);
 
 #endif /* WEECHAT_PLUGIN_LOGGER_INFO_H */
