@@ -1,7 +1,7 @@
 /*
- * relay-remote-event.c - process events received from relay remote
+ * SPDX-FileCopyrightText: 2024-2025 Sébastien Helleu <flashcode@flashtux.org>
  *
- * Copyright (C) 2024 Sébastien Helleu <flashcode@flashtux.org>
+ * SPDX-License-Identifier: GPL-3.0-or-later
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -19,6 +19,8 @@
  * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+/* Process events received from relay remote */
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
@@ -156,7 +158,7 @@ relay_remote_build_string_tags (cJSON *json_tags, int line_id, int highlight)
             ptr_tag = cJSON_GetStringValue (json_tag);
             if (ptr_tag)
             {
-                if (*tags[0])
+                if ((*tags)[0])
                     weechat_string_dyn_concat (tags, ",", -1);
                 if (highlight && (strncmp (ptr_tag, "notify_", 7) == 0))
                 {
@@ -177,14 +179,14 @@ relay_remote_build_string_tags (cJSON *json_tags, int line_id, int highlight)
      */
     if (highlight && !tag_notify_highlight)
     {
-        if (*tags[0])
+        if ((*tags)[0])
             weechat_string_dyn_concat (tags, ",", -1);
         weechat_string_dyn_concat (tags, "notify_highlight", -1);
     }
 
     /* add tag with remote line id */
     snprintf (str_tag_id, sizeof (str_tag_id), "relay_remote_line_id_%d", line_id);
-    if (*tags[0])
+    if ((*tags)[0])
         weechat_string_dyn_concat (tags, ",", -1);
     weechat_string_dyn_concat (tags, str_tag_id, -1);
 
@@ -267,9 +269,8 @@ relay_remote_event_search_line_by_id (struct t_gui_buffer *buffer, int id)
     struct t_gui_lines *ptr_lines;
     struct t_gui_line *ptr_line;
     struct t_gui_line_data *ptr_line_data;
-    const char **tags;
+    const char **tags, **ptr_tag;
     char str_tag_id[512];
-    int i;
 
     if (!buffer)
         return NULL;
@@ -292,9 +293,9 @@ relay_remote_event_search_line_by_id (struct t_gui_buffer *buffer, int id)
         tags = weechat_hdata_pointer (relay_hdata_line_data, ptr_line_data, "tags_array");
         if (tags)
         {
-            for (i = 0; tags[i]; i++)
+            for (ptr_tag = tags; *ptr_tag; ptr_tag++)
             {
-                if (weechat_strcmp (tags[i], str_tag_id) == 0)
+                if (weechat_strcmp (*ptr_tag, str_tag_id) == 0)
                     return ptr_line;
             }
         }
@@ -360,14 +361,14 @@ relay_remote_event_line_update (struct t_relay_remote_event *event)
     if (!hashtable)
         return;
 
-    snprintf (str_value, sizeof (str_value), "%ld", tv_date.tv_sec);
+    snprintf (str_value, sizeof (str_value), "%lld", (long long)tv_date.tv_sec);
     weechat_hashtable_set (hashtable, "date", str_value);
-    snprintf (str_value, sizeof (str_value), "%ld", tv_date.tv_usec);
+    snprintf (str_value, sizeof (str_value), "%ld", (long)tv_date.tv_usec);
     weechat_hashtable_set (hashtable, "date_usec", str_value);
 
-    snprintf (str_value, sizeof (str_value), "%ld", tv_date_printed.tv_sec);
+    snprintf (str_value, sizeof (str_value), "%lld", (long long)tv_date_printed.tv_sec);
     weechat_hashtable_set (hashtable, "date_printed", str_value);
-    snprintf (str_value, sizeof (str_value), "%ld", tv_date_printed.tv_usec);
+    snprintf (str_value, sizeof (str_value), "%ld", (long)tv_date_printed.tv_usec);
     weechat_hashtable_set (hashtable, "date_usec_printed", str_value);
 
     tags = relay_remote_build_string_tags (
@@ -1103,6 +1104,64 @@ relay_remote_event_clear_buffers (struct t_relay_remote *remote)
     }
 }
 
+/*
+ * Synchronizes with remote by sending two requests:
+ *   - "GET /api/buffers": fetch buffers data
+ *   - "POST /api/sync": synchronize with remote to receive events
+ */
+
+void
+relay_remote_event_sync_with_remote (struct t_relay_remote *remote)
+{
+    cJSON *json, *json_req1, *json_req2, *json_body;
+    char url[1024];
+
+    if (!remote)
+        return;
+
+    json = cJSON_CreateArray ();
+    if (!json)
+        goto end;
+
+    /* first request: GET /api/buffers */
+    json_req1 = cJSON_CreateObject ();
+    if (json_req1)
+    {
+        snprintf (url, sizeof (url),
+                  "GET /api/buffers?lines=-%d&nicks=true&colors=weechat",
+                  weechat_config_integer (relay_config_api_remote_get_lines));
+        cJSON_AddItemToObject (json_req1, "request", cJSON_CreateString (url));
+        cJSON_AddItemToObject (
+            json_req1,
+            "request_id",
+            cJSON_CreateString (RELAY_REMOTE_EVENT_ID_INITIAL_SYNC));
+        cJSON_AddItemToArray (json, json_req1);
+    }
+
+    /* second request: POST /api/sync */
+    json_req2 = cJSON_CreateObject ();
+    if (json_req2)
+    {
+        cJSON_AddItemToObject (json_req2, "request",
+                               cJSON_CreateString ("POST /api/sync"));
+        json_body = cJSON_CreateObject ();
+        if (!json_body)
+        {
+            cJSON_Delete (json_req2);
+            goto end;
+        }
+        cJSON_AddItemToObject (json_body, "colors",
+                               cJSON_CreateString ("weechat"));
+        cJSON_AddItemToObject (json_req2, "body", json_body);
+        cJSON_AddItemToArray (json, json_req2);
+    }
+
+    relay_remote_network_send_json (remote, json);
+
+end:
+    cJSON_Delete (json);
+}
+
 /*
  * Callback for response to GET /api/version.
  */
@@ -1111,7 +1170,7 @@ RELAY_REMOTE_EVENT_CALLBACK(version)
 {
     cJSON *json_obj;
     const char *weechat_version, *weechat_version_git, *relay_api_version;
-    char *weechat_version_local, request[1024];
+    char *weechat_version_local;
 
     if (!event || !event->json)
         return WEECHAT_RC_OK;
@@ -1148,18 +1207,9 @@ RELAY_REMOTE_EVENT_CALLBACK(version)
                 relay_remote_network_disconnect (event->remote);
                 return WEECHAT_RC_OK;
         }
-        relay_remote_event_clear_buffers (event->remote);
         event->remote->version_ok = 1;
-        snprintf (
-            request, sizeof (request),
-            "{"
-            "\"request\": "
-            "\"GET /api/buffers?lines=-%d&nicks=true&colors=weechat\", "
-            "\"request_id\": \"" RELAY_REMOTE_EVENT_ID_INITIAL_SYNC "\""
-            "}",
-            weechat_config_integer (relay_config_api_remote_get_lines));
-        relay_remote_network_send (event->remote, RELAY_MSG_STANDARD,
-                                   request, strlen (request));
+        relay_remote_event_clear_buffers (event->remote);
+        relay_remote_event_sync_with_remote (event->remote);
     }
 
     return WEECHAT_RC_OK;
@@ -1191,40 +1241,6 @@ RELAY_REMOTE_EVENT_CALLBACK(quit)
     return WEECHAT_RC_OK;
 }
 
-/*
- * Synchronizes with remote.
- */
-
-void
-relay_remote_event_sync_with_remote (struct t_relay_remote *remote)
-{
-    cJSON *json, *json_body;
-
-    if (!remote)
-        return;
-
-    json = cJSON_CreateObject ();
-    if (!json)
-        goto end;
-
-    cJSON_AddItemToObject (json, "request",
-                           cJSON_CreateString ("POST /api/sync"));
-    json_body = cJSON_CreateObject ();
-    if (!json_body)
-        goto end;
-
-    cJSON_AddItemToObject (json_body, "colors",
-                           cJSON_CreateString ("weechat"));
-    cJSON_AddItemToObject (json, "body", json_body);
-
-    relay_remote_network_send_json (remote, json);
-
-    remote->synced = 1;
-
-end:
-    cJSON_Delete (json);
-}
-
 /*
  * Reads an event from a remote.
  */
@@ -1235,7 +1251,7 @@ relay_remote_event_recv (struct t_relay_remote *remote, const char *data)
     cJSON *json, *json_body, *json_obj;
     const char *body_type, *event_name, *request_id;
     long long buffer_id;
-    int i, rc, code;
+    int i, rc, code, initial_sync;
     struct t_relay_remote_event_cb event_cb[] = {
         /* event (mask), callback (NULL = event ignored) */
         /* note: order is important */
@@ -1313,15 +1329,15 @@ relay_remote_event_recv (struct t_relay_remote *remote, const char *data)
         }
     }
 
+    initial_sync = (weechat_strcmp (request_id,
+                                    RELAY_REMOTE_EVENT_ID_INITIAL_SYNC) == 0) ?
+        1 : 0;
+
     if (callback)
     {
         event.json = json_body;
-        if ((weechat_strcmp (body_type, "buffers") == 0)
-            && (weechat_strcmp (request_id,
-                                RELAY_REMOTE_EVENT_ID_INITIAL_SYNC) == 0))
-        {
+        if ((weechat_strcmp (body_type, "buffers") == 0) && initial_sync)
             relay_remote_event_initial_sync_buffers (&event);
-        }
         rc = WEECHAT_RC_OK;
         if (cJSON_IsArray (json_body))
         {
@@ -1339,12 +1355,10 @@ relay_remote_event_recv (struct t_relay_remote *remote, const char *data)
             goto error_cb;
     }
 
-    if (!remote->synced
-        && (code == 200)
-        && ((weechat_strcmp (body_type, "buffers") == 0)
-            || (weechat_strcmp (body_type, "buffer") == 0)))
+    if (!remote->synced && initial_sync)
     {
-        relay_remote_event_sync_with_remote (remote);
+        remote->synced = 1;
+        weechat_bar_item_update ("input_prompt");
     }
 
     return;
