@@ -38,6 +38,33 @@ extern "C"
 #include "src/core/core-util.h"
 }
 
+#define WEE_PARSE_NUMBER(__result, __parsed, __string, __base) \
+    LONGS_EQUAL(                                               \
+        __result,                                              \
+        util_parse_int (__string, __base, NULL));              \
+    LONGS_EQUAL(                                               \
+        __result,                                              \
+        util_parse_int (__string, __base, &number_int));       \
+    if (__result)                                              \
+        LONGS_EQUAL(__parsed, number_int);                     \
+    LONGS_EQUAL(                                               \
+        __result,                                              \
+        util_parse_long (__string, __base, NULL));             \
+    LONGS_EQUAL(                                               \
+        __result,                                              \
+        util_parse_long (__string, __base, &number_long));     \
+    if (__result)                                              \
+        LONGS_EQUAL(__parsed, number_long);                    \
+    LONGS_EQUAL(                                               \
+        __result,                                              \
+        util_parse_longlong (__string, __base, NULL));         \
+    LONGS_EQUAL(                                               \
+        __result,                                              \
+        util_parse_longlong (__string, __base,                 \
+                             &number_longlong));               \
+    if (__result)                                              \
+        CHECK_EQUAL(__parsed, number_longlong);
+
 #define WEE_PARSE_DATE(__result, __sec, __usec, __datetime)    \
     tv.tv_sec = 0;                                             \
     tv.tv_usec = 0;                                            \
@@ -56,6 +83,101 @@ TEST_GROUP(CoreUtil)
 {
 };
 
+/*
+ * Tests functions:
+ *   util_parse_int
+ *   util_parse_long
+ *   util_parse_longlong
+ */
+
+TEST(CoreUtil, ParseNumber)
+{
+    int number_int;
+    long number_long;
+    long long number_longlong;
+    char str_number[256];
+
+    /* NULL string */
+    WEE_PARSE_NUMBER(0, 0, NULL, 10);
+
+    /* invalid base */
+    WEE_PARSE_NUMBER(0, 0, "123", -1);
+    WEE_PARSE_NUMBER(0, 0, "123", 1);
+    WEE_PARSE_NUMBER(0, 0, "123", 37);
+
+    /* invalid number */
+    WEE_PARSE_NUMBER(0, 0, "", 10);
+    WEE_PARSE_NUMBER(0, 0, " ", 10);
+    WEE_PARSE_NUMBER(0, 0, "-", 10);
+    WEE_PARSE_NUMBER(0, 0, "+", 10);
+    WEE_PARSE_NUMBER(0, 0, "3-", 10);
+    WEE_PARSE_NUMBER(0, 0, "3+", 10);
+    WEE_PARSE_NUMBER(0, 0, "--3", 10);
+    WEE_PARSE_NUMBER(0, 0, "++3", 10);
+    WEE_PARSE_NUMBER(0, 0, ".1", 10);
+    WEE_PARSE_NUMBER(0, 0, "1.", 10);
+    WEE_PARSE_NUMBER(0, 0, "1.2", 10);
+    WEE_PARSE_NUMBER(0, 0, "1,2", 10);
+    WEE_PARSE_NUMBER(0, 0, "a", 10);
+    WEE_PARSE_NUMBER(0, 0, "1a", 10);
+    WEE_PARSE_NUMBER(0, 0, "a1", 10);
+    WEE_PARSE_NUMBER(0, 0, "123 ", 10);
+    WEE_PARSE_NUMBER(0, 0, "12", 2);
+    WEE_PARSE_NUMBER(0, 0, "18", 8);
+    WEE_PARSE_NUMBER(0, 0, "1g", 16);
+
+    /* invalid int: outside range (INT_MIN, INT_MAX) */
+    snprintf (str_number, sizeof (str_number), "%d1", INT_MIN);
+    LONGS_EQUAL(0, util_parse_int (str_number, 10, &number_int));
+    snprintf (str_number, sizeof (str_number), "%d1", INT_MAX);
+    LONGS_EQUAL(0, util_parse_int (str_number, 10, &number_int));
+
+    /* invalid long: outside range (LONG_MIN, LONG_MAX) */
+    snprintf (str_number, sizeof (str_number), "%ld1", LONG_MIN);
+    LONGS_EQUAL(0, util_parse_long (str_number, 10, &number_long));
+    snprintf (str_number, sizeof (str_number), "%ld1", LONG_MAX);
+    LONGS_EQUAL(0, util_parse_long (str_number, 10, &number_long));
+
+    /* invalid long long: outside range (LLONG_MIN, LLONG_MAX) */
+    snprintf (str_number, sizeof (str_number), "%lld1", LLONG_MIN);
+    LONGS_EQUAL(0, util_parse_longlong (str_number, 10, &number_longlong));
+    snprintf (str_number, sizeof (str_number), "%lld1", LLONG_MAX);
+    LONGS_EQUAL(0, util_parse_longlong (str_number, 10, &number_longlong));
+
+    /* OK */
+    WEE_PARSE_NUMBER(1, 12, "12", 10);
+    WEE_PARSE_NUMBER(1, 15, "  15", 10);
+    WEE_PARSE_NUMBER(1, -3, "-3", 10);
+    WEE_PARSE_NUMBER(1, 5, "101", 2);
+    WEE_PARSE_NUMBER(1, 71, "107", 8);
+    WEE_PARSE_NUMBER(1, 30, "1e", 16);
+    WEE_PARSE_NUMBER(1, 31, "1F", 16);
+
+    /* OK, boundary limits for int */
+    snprintf (str_number, sizeof (str_number), "%d", INT_MIN);
+    LONGS_EQUAL(1, util_parse_int (str_number, 10, &number_int));
+    LONGS_EQUAL(INT_MIN, number_int);
+    snprintf (str_number, sizeof (str_number), "%d", INT_MAX);
+    LONGS_EQUAL(1, util_parse_int (str_number, 10, &number_int));
+    LONGS_EQUAL(INT_MAX, number_int);
+
+    /* OK, boundary limits for long */
+    snprintf (str_number, sizeof (str_number), "%ld", LONG_MIN);
+    LONGS_EQUAL(1, util_parse_long (str_number, 10, &number_long));
+    LONGS_EQUAL(LONG_MIN, number_long);
+    snprintf (str_number, sizeof (str_number), "%ld", LONG_MAX);
+    LONGS_EQUAL(1, util_parse_long (str_number, 10, &number_long));
+    LONGS_EQUAL(LONG_MAX, number_long);
+
+    /* OK, boundary limits for long long */
+    snprintf (str_number, sizeof (str_number), "%lld", LLONG_MIN);
+    LONGS_EQUAL(1, util_parse_longlong (str_number, 10, &number_longlong));
+    CHECK_EQUAL(LLONG_MIN, number_longlong);
+    snprintf (str_number, sizeof (str_number), "%lld", LLONG_MAX);
+    LONGS_EQUAL(1, util_parse_longlong (str_number, 10, &number_longlong));
+    CHECK_EQUAL(LLONG_MAX, number_longlong);
+}
+
 /*
  * Tests functions:
  *   util_timeval_cmp
@@ -291,8 +413,9 @@ TEST(CoreUtil, ParseTime)
     time_t date;
     char str_time[128];
 
-    /* test with local timezone: UTC+1 */
-    setenv ("TZ", "UTC+1", 1);
+    /* test with UTC timezone */
+    setenv ("TZ", "UTC", 1);
+    tzset ();
 
     LONGS_EQUAL(0, util_parse_time (NULL, NULL));
     LONGS_EQUAL(0, util_parse_time (NULL, &tv));
@@ -300,11 +423,13 @@ TEST(CoreUtil, ParseTime)
     WEE_PARSE_DATE(0, 0, 0, "");
     WEE_PARSE_DATE(0, 0, 0, "invalid");
 
+    /* invalid: negative microseconds */
+    WEE_PARSE_DATE(0, 0, 0, "1703500149.-456789");
+
     /*
      * expected: 2023-12-25T00:00:00Z == 1703462400
-     * (local timezone UTC+1: 1703466000)
      */
-    WEE_PARSE_DATE(1, 1703466000, 0, "2023-12-25");
+    WEE_PARSE_DATE(1, 1703462400, 0, "2023-12-25");
 
     /* expected: current date with specified local time */
     date = time (NULL);
@@ -321,29 +446,83 @@ TEST(CoreUtil, ParseTime)
     CHECK((tv.tv_sec >= date) && (tv.tv_sec <= date + 10));
     LONGS_EQUAL(456789, tv.tv_usec);
 
-    /* expected: current date with specified UTC time */
+    /* expected: current date with specified local time + timezone offset */
     date = time (NULL);
     local_time = localtime (&date);
-    strftime (str_time, sizeof (str_time), "%H:%M:%SZ", local_time);
+    strftime (str_time, sizeof (str_time), "%H:%M:%S+0100", local_time);
     LONGS_EQUAL(1, util_parse_time (str_time, &tv));
     CHECK((tv.tv_sec >= date - 3600) && (tv.tv_sec <= date - 3600 + 10));
     LONGS_EQUAL(0, tv.tv_usec);
 
     date = time (NULL);
     local_time = localtime (&date);
-    strftime (str_time, sizeof (str_time), "%H:%M:%S.456789Z", local_time);
+    strftime (str_time, sizeof (str_time), "%H:%M:%S +01:00", local_time);
+    LONGS_EQUAL(1, util_parse_time (str_time, &tv));
+    CHECK((tv.tv_sec >= date - 3600) && (tv.tv_sec <= date - 3600 + 10));
+    LONGS_EQUAL(0, tv.tv_usec);
+
+    date = time (NULL);
+    local_time = localtime (&date);
+    strftime (str_time, sizeof (str_time), "%H:%M:%S +02:00", local_time);
+    LONGS_EQUAL(1, util_parse_time (str_time, &tv));
+    CHECK((tv.tv_sec >= date - 7200) && (tv.tv_sec <= date - 7200 + 10));
+    LONGS_EQUAL(0, tv.tv_usec);
+
+    date = time (NULL);
+    local_time = localtime (&date);
+    strftime (str_time, sizeof (str_time), "%H:%M:%S.456789+0100", local_time);
     LONGS_EQUAL(1, util_parse_time (str_time, &tv));
     CHECK((tv.tv_sec >= date - 3600) && (tv.tv_sec <= date - 3600 + 10));
     LONGS_EQUAL(456789, tv.tv_usec);
 
+    date = time (NULL);
+    local_time = localtime (&date);
+    strftime (str_time, sizeof (str_time), "%H:%M:%S.456789 +01:00", local_time);
+    LONGS_EQUAL(1, util_parse_time (str_time, &tv));
+    CHECK((tv.tv_sec >= date - 3600) && (tv.tv_sec <= date - 3600 + 10));
+    LONGS_EQUAL(456789, tv.tv_usec);
+
+    date = time (NULL);
+    local_time = localtime (&date);
+    strftime (str_time, sizeof (str_time), "%H:%M:%S.456789 +02:00", local_time);
+    LONGS_EQUAL(1, util_parse_time (str_time, &tv));
+    CHECK((tv.tv_sec >= date - 7200) && (tv.tv_sec <= date - 7200 + 10));
+    LONGS_EQUAL(456789, tv.tv_usec);
+
+    /* expected: current date with specified UTC time */
+    date = time (NULL);
+    local_time = localtime (&date);
+    strftime (str_time, sizeof (str_time), "%H:%M:%SZ", local_time);
+    LONGS_EQUAL(1, util_parse_time (str_time, &tv));
+    CHECK((tv.tv_sec >= date) && (tv.tv_sec <= date + 10));
+    LONGS_EQUAL(0, tv.tv_usec);
+
+    date = time (NULL);
+    local_time = localtime (&date);
+    strftime (str_time, sizeof (str_time), "%H:%M:%S.456789Z", local_time);
+    LONGS_EQUAL(1, util_parse_time (str_time, &tv));
+    CHECK((tv.tv_sec >= date) && (tv.tv_sec <= date + 10));
+    LONGS_EQUAL(456789, tv.tv_usec);
+
     /*
      * expected: 2023-12-25T10:29:09.456789Z == 1703500149.456789
-     * (local timezone UTC+1: 1703503749.456789)
      */
-    WEE_PARSE_DATE(1, 1703500149 + 3600, 0, "2023-12-25T10:29:09");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25T10:29:09");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25t10:29:09");
     WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25T10:29:09Z");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25t10:29:09z");
     WEE_PARSE_DATE(1, 1703500149, 456000, "2023-12-25T10:29:09.456Z");
     WEE_PARSE_DATE(1, 1703500149, 456789, "2023-12-25T10:29:09.456789Z");
+    WEE_PARSE_DATE(1, 1703500149, 456789, "2023-12-25t10:29:09.456789z");
+
+    /*
+     * expected: 2023-12-25T10:29:09.456789Z == 1703500149.456789
+     * with space instead of "T"
+     */
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09Z");
+    WEE_PARSE_DATE(1, 1703500149, 456000, "2023-12-25 10:29:09.456Z");
+    WEE_PARSE_DATE(1, 1703500149, 456789, "2023-12-25 10:29:09.456789Z");
 
     /*
      * expected: 2023-12-25T10:29:09.456789Z == 1703500149.456789
@@ -357,20 +536,74 @@ TEST(CoreUtil, ParseTime)
     WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25T10:29:09-00:00");
     WEE_PARSE_DATE(1, 1703500149, 456789, "2023-12-25T10:29:09.456789-00:00");
 
-    WEE_PARSE_DATE(1, 1703500149 + 7200, 0, "2023-12-25T10:29:09+02");
-    WEE_PARSE_DATE(1, 1703500149 + 60, 0, "2023-12-25T10:29:09+0001");
-    WEE_PARSE_DATE(1, 1703500149 + 60, 456789, "2023-12-25T10:29:09.456789+0001");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25T12:29:09+02");
+    WEE_PARSE_DATE(1, 1703500149 - 60, 0, "2023-12-25T10:29:09+0001");
+    WEE_PARSE_DATE(1, 1703500149 - 60, 456789, "2023-12-25T10:29:09.456789+0001");
 
-    WEE_PARSE_DATE(1, 1703500149 + 7200, 0, "2023-12-25T10:29:09+0200");
-    WEE_PARSE_DATE(1, 1703500149 + 7200, 0, "2023-12-25T10:29:09+02:00");
-    WEE_PARSE_DATE(1, 1703500149 + 3600 + 1800, 0, "2023-12-25T10:29:09+0130");
-    WEE_PARSE_DATE(1, 1703500149 + 3600 + 1800, 0, "2023-12-25T10:29:09+01:30");
-    WEE_PARSE_DATE(1, 1703500149 + 3600 + 1800, 456789, "2023-12-25T10:29:09.456789+01:30");
+    WEE_PARSE_DATE(1, 1703500149 - 7200, 0, "2023-12-25T10:29:09+0200");
+    WEE_PARSE_DATE(1, 1703500149 - 7200, 0, "2023-12-25T10:29:09+02:00");
+    WEE_PARSE_DATE(1, 1703500149 - 3600 - 1800, 0, "2023-12-25T10:29:09+0130");
+    WEE_PARSE_DATE(1, 1703500149 - 3600 - 1800, 0, "2023-12-25T10:29:09+01:30");
+    WEE_PARSE_DATE(1, 1703500149 - 3600 - 1800, 456789, "2023-12-25T10:29:09.456789+01:30");
 
-    WEE_PARSE_DATE(1, 1703500149 - 60, 0, "2023-12-25T10:29:09-0001");
-    WEE_PARSE_DATE(1, 1703500149 - 3600 - 1800, 0, "2023-12-25T10:29:09-0130");
-    WEE_PARSE_DATE(1, 1703500149 - 3600 - 1800, 0, "2023-12-25T10:29:09-01:30");
-    WEE_PARSE_DATE(1, 1703500149 - 3600 - 1800, 456789, "2023-12-25T10:29:09.456789-01:30");
+    WEE_PARSE_DATE(1, 1703500149 + 60, 0, "2023-12-25T10:29:09-0001");
+    WEE_PARSE_DATE(1, 1703500149 + 3600 + 1800, 0, "2023-12-25T10:29:09-0130");
+    WEE_PARSE_DATE(1, 1703500149 + 3600 + 1800, 0, "2023-12-25T10:29:09-01:30");
+    WEE_PARSE_DATE(1, 1703500149 + 3600 + 1800, 456789, "2023-12-25T10:29:09.456789-01:30");
+
+    /*
+     * expected: 2023-12-25T10:29:09.456789Z == 1703500149.456789
+     * with space instead of "T" and timezone offset
+     */
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09+00");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09+0000");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09+00:00");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09-00");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09-0000");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09-00:00");
+    WEE_PARSE_DATE(1, 1703500149, 456789, "2023-12-25 10:29:09.456789-00:00");
+
+    WEE_PARSE_DATE(1, 1703500149 - 7200, 0, "2023-12-25 10:29:09+02");
+    WEE_PARSE_DATE(1, 1703500149 - 60, 0, "2023-12-25 10:29:09+0001");
+    WEE_PARSE_DATE(1, 1703500149 - 60, 456789, "2023-12-25 10:29:09.456789+0001");
+
+    WEE_PARSE_DATE(1, 1703500149 - 7200, 0, "2023-12-25 10:29:09+0200");
+    WEE_PARSE_DATE(1, 1703500149 - 7200, 0, "2023-12-25 10:29:09+02:00");
+    WEE_PARSE_DATE(1, 1703500149 - 3600 - 1800, 0, "2023-12-25 10:29:09+0130");
+    WEE_PARSE_DATE(1, 1703500149 - 3600 - 1800, 0, "2023-12-25 10:29:09+01:30");
+    WEE_PARSE_DATE(1, 1703500149 - 3600 - 1800, 456789, "2023-12-25 10:29:09.456789+01:30");
+
+    WEE_PARSE_DATE(1, 1703500149 + 60, 0, "2023-12-25 10:29:09-0001");
+    WEE_PARSE_DATE(1, 1703500149 + 3600 + 1800, 0, "2023-12-25 10:29:09-0130");
+    WEE_PARSE_DATE(1, 1703500149 + 3600 + 1800, 0, "2023-12-25 10:29:09-01:30");
+    WEE_PARSE_DATE(1, 1703500149 + 3600 + 1800, 456789, "2023-12-25 10:29:09.456789-01:30");
+
+    /*
+     * expected: 2023-12-25T10:29:09.456789Z == 1703500149.456789
+     * with space instead of "T" and timezone offset after a space
+     */
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09 +00");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09 +0000");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09 +00:00");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09 -00");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09 -0000");
+    WEE_PARSE_DATE(1, 1703500149, 0, "2023-12-25 10:29:09 -00:00");
+    WEE_PARSE_DATE(1, 1703500149, 456789, "2023-12-25 10:29:09.456789 -00:00");
+
+    WEE_PARSE_DATE(1, 1703500149 - 7200, 0, "2023-12-25 10:29:09 +02");
+    WEE_PARSE_DATE(1, 1703500149 - 60, 0, "2023-12-25 10:29:09 +0001");
+    WEE_PARSE_DATE(1, 1703500149 - 60, 456789, "2023-12-25 10:29:09.456789 +0001");
+
+    WEE_PARSE_DATE(1, 1703500149 - 7200, 0, "2023-12-25 10:29:09 +0200");
+    WEE_PARSE_DATE(1, 1703500149 - 7200, 0, "2023-12-25 10:29:09 +02:00");
+    WEE_PARSE_DATE(1, 1703500149 - 3600 - 1800, 0, "2023-12-25 10:29:09 +0130");
+    WEE_PARSE_DATE(1, 1703500149 - 3600 - 1800, 0, "2023-12-25 10:29:09 +01:30");
+    WEE_PARSE_DATE(1, 1703500149 - 3600 - 1800, 456789, "2023-12-25 10:29:09.456789 +01:30");
+
+    WEE_PARSE_DATE(1, 1703500149 + 60, 0, "2023-12-25 10:29:09 -0001");
+    WEE_PARSE_DATE(1, 1703500149 + 3600 + 1800, 0, "2023-12-25 10:29:09 -0130");
+    WEE_PARSE_DATE(1, 1703500149 + 3600 + 1800, 0, "2023-12-25 10:29:09 -01:30");
+    WEE_PARSE_DATE(1, 1703500149 + 3600 + 1800, 456789, "2023-12-25 10:29:09.456789 -01:30");
 
     /* expected: 2023-12-25T10:29:09.456789Z == 1703500149.456789 */
     WEE_PARSE_DATE(1, 1703500149, 0, "1703500149");
@@ -379,7 +612,13 @@ TEST(CoreUtil, ParseTime)
     WEE_PARSE_DATE(1, 1703500149, 456000, "1703500149,456");
     WEE_PARSE_DATE(1, 1703500149, 456789, "1703500149,456789");
 
+    /* expected: 2023-12-25T10:29:09.456789Z == 1703500149.456789 */
+    /* with extra digits after microseconds */
+    WEE_PARSE_DATE(1, 1703500149, 456789, "1703500149.4567891");
+    WEE_PARSE_DATE(1, 1703500149, 456789, "1703500149.456789123456789123456789123456789");
+
     setenv ("TZ", "", 1);
+    tzset ();
 }
 
 /*
